/*

                                   _JNJ`                        
                               .JNMH`                           
                              JMMF`       `;.                   
                            .NMM)          `MN.                 
                            MMM)            (MML                
                           (MMM`             MMML               
                     M  I  D  N  I  G  H  T    C  o  D  E       
                           (NMMF             MHNH               
                            NMML            .MMM                
                             NMML          .NMH                 
                              4MMNL       .#F                   
                               `4HNNL_   `                      
                                   `"""`                        

                          Copyright (C) 2004-2015
       "Ian (Larry) Latter" <ian dot latter at midnightcode dot org>

    Midnight Code is a registered trademark of Ian Latter.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, and mirrored at the Midnight Code web
    site; as at version 2 of the License only.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    (version 2) along with this program; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA, or see http://midnightcode.org/gplv2.txt

*/

/*

  Hardware;

    JayCar (Australia) for misc electronic components;
      EtherTen:           http://jaycar.com.au/productView.asp?ID=XC4216
      Temperature Sensor: http://jaycar.com.au/productView.asp?ID=XC4230
      Header Strip:       http://jaycar.com.au/productView.asp?ID=HM3212
      Rainbow Cable:      http://jaycar.com.au/productView.asp?ID=WM4516
      
*/


#include "Arduino.h"
#include <SPI.h>
#include <Ethernet.h>

#define REF_PIN 2
void getCurrentTemp( int *sign, int *whole, int *fract);
char temp_string[10];
double temp_value;

byte mac[] = { 0xC0, 0xDE, 0x00, 0x00, 0x00, 0x02 };

const char *server = "cloud.nimbits.com";
String path = "/service/v2/value";
String email = "user_email@addr.ess";
String key = "user_key"; 
char *point = "temperature";
int port = 80;

EthernetClient client;


void setup() {
  Serial.begin(9600);

  // initialize DS18B20 datapin
  Serial.print("Init sensor ..");
  digitalWrite(REF_PIN, LOW);
  pinMode(REF_PIN, INPUT);      // sets the digital pin as input (logic 1)
  pinMode(15, INPUT);
  Serial.println(".. sensor ready");

  Serial.print("Init network ..");
  Ethernet.begin(mac);
  Serial.print(".. network ready (");
  Serial.print(Ethernet.localIP());
  Serial.println(")");
}


void loop() {
  getCurrentTemp(temp_string);
  Serial.println(temp_string);
  temp_value = atof(temp_string);
  recordValue(temp_value, point);
  delay(5000);
}


void recordValue(double value, char *pointId) {
  EthernetClient client;
  String json;
  String content;

  json =  "{\"d\":\"";
  json += doubleToString(value, 2);
  json += "\"}"; 

  content = "email=";
  content += email;
  content += "&key=";
  content += key;
  content += "&id=";
  content += email;
  content += "/";
  content += pointId;
  content += "&json=";
  content += json;

  Serial.println(content);

  if(client.connect(server, port)) {
    client.print("POST ");
    client.print(path);
    client.println(" HTTP/1.1");
    client.print("Host: ");
    client.println(server);
    client.println("Connection: close");
    client.println("User-Agent: Arduino/1.0.4");
    client.println("Cache-Control: max-age=0");
    client.println("Content-Type: application/x-www-form-urlencoded");
    client.print("Content-Length: ");
    client.println(content.length());
    client.println();
    client.println(content);

    while(client.connected() && !client.available()) 
      delay(1);
    while(client.available() ) {
      char c = client.read();
      Serial.print(c);
    }
  }
}


// Rounds down (via intermediary integer conversion truncation)
String doubleToString(double input,int decimalPlaces){
  String string;

  if(decimalPlaces!=0){
    string = String((int)(input*pow(10,decimalPlaces)));
    if(abs(input) < 1){
      if(input > 0)
        string = "0"+string;
      else if(input < 0)
        string = string.substring(0,1)+"0"+string.substring(1);
      }
      return string.substring(0,string.length()-decimalPlaces)+"."+string.substring(string.length()-decimalPlaces);
    }
  else {
    return String((int)input);
  }
}


void OneWireReset (int Pin) // reset.  Should improve to act as a presence pulse
{
  digitalWrite (Pin, LOW);
  pinMode (Pin, OUTPUT);        // bring low for 500 us
  delayMicroseconds (500);
  pinMode (Pin, INPUT);
  delayMicroseconds (500);
}


void OneWireOutByte (int Pin, byte d) // output byte d (least sig bit first).
{
  byte n;

  for (n = 8; n != 0; n--)
  {
    if ((d & 0x01) == 1)  // test least sig bit
    {
      digitalWrite (Pin, LOW);
      pinMode (Pin, OUTPUT);
      delayMicroseconds (5);
      pinMode (Pin, INPUT);
      delayMicroseconds (60);
    }
    else
    {
      digitalWrite (Pin, LOW);
      pinMode (Pin, OUTPUT);
      delayMicroseconds (60);
      pinMode (Pin, INPUT);
    }

    d = d >> 1; // now the next bit is in the least sig bit position.
  }
}


byte OneWireInByte (int Pin) // read byte, least sig byte first
{
  byte d, n, b;

  for (n = 0; n < 8; n++)
  {
    digitalWrite (Pin, LOW);
    pinMode (Pin, OUTPUT);
    delayMicroseconds (5);
    pinMode (Pin, INPUT);
    delayMicroseconds (5);
    b = digitalRead (Pin);
    delayMicroseconds (50);
    d = (d >> 1) | (b << 7); // shift d to right and insert b in most sig bit position
  }
  return (d);
}


void getCurrentTemp (char *temp)
{
  int HighByte, LowByte, TReading, Tc_100, sign, whole, fract;

  OneWireReset (REF_PIN);
  OneWireOutByte (REF_PIN, 0xcc);
  OneWireOutByte (REF_PIN, 0x44); // perform temperature conversion, strong pullup for one sec

  OneWireReset (REF_PIN);
  OneWireOutByte (REF_PIN, 0xcc);
  OneWireOutByte (REF_PIN, 0xbe);

  LowByte = OneWireInByte (REF_PIN);
  HighByte = OneWireInByte (REF_PIN);
  TReading = (HighByte << 8) + LowByte;
  sign = TReading & 0x8000;  // test most sig bit
  if (sign) // negative
  {
    TReading = (TReading ^ 0xffff) + 1; // 2's comp
  }
  Tc_100 = (6 * TReading) + TReading / 4; // multiply by (100 * 0.0625) or 6.25

  whole = Tc_100 / 100;  // separate off the whole and fractional portions
  fract = Tc_100 % 100;

  if (sign) {
    temp[0] = '-';
  } else {
    temp[0] = '+';
  }

  if (whole / 100 == 0) {
    temp[1] = '0';
  } else {
    temp[1] = whole / 100 + '0';
  }

  temp[2] = (whole - (whole / 100) * 100) / 10 + '0' ;
  temp[3] = whole - (whole / 10) * 10 + '0';
  temp[4] = '.';
  temp[5] = fract / 10 + '0';
  temp[6] = fract - (fract / 10) * 10 + '0';
  temp[7] = '\0';
}

